
  rm(list=ls())
  # load the libary
  library(IsoMSTOF)

  #-------------------------------Start of parameter table--------------------------------------#

  peaklistDataFolder<-"./peaklistData/"           # Assign a folder for raw data
  convertedDataFolder<-"./convertedPeaklist/"     # Assign a folder for converted peak lists
  dataSaveFolder<-"./outputPairlist/"             # Assign a folder for detected pair lists               
  companyNO<-1                                    # Instrument manufacturer NO: Agilent=1, Bruker=2,Waters=3.
  
  dataSaveFolderDebug<-paste(dataSaveFolder,"debug/",sep="")  

  if (companyNO==1) {              
     AgilentDataConvert(peaklistDataFolder,convertedDataFolder)
     dataFolder<-"./convertedPeaklist/"
  }  else if (companyNO==3) {         
     WatersDataConvert(peaklistData,convertedDataFolder)
     dataFolder<-"./convertedPeaklist/"
  }  else {          
     dataFolder<-"./peaklistData/"         
  }      
 
  isFT<-FALSE                        
  rtWinMin<-100                      # Starting time of chromatography that will be processed.
  rtWinMax<-1800                     # End time of chromatography that will be processed.
  mzWinMin<-265                      # Minimal m/z that will be processed.
  mzWinMax<-1000                     # Maximal m/z that will be processed.
  intThreshold<-1500                 # Threshold that cuts off low abundant features.  
  isFTHomo<-FALSE                    
  isTOFTailing<-FALSE                
  
  if (isFT==TRUE){
     minResRatio<-0.15
     if (isFTHomo==TRUE){
     mzWinHomo<-0.25                     
     mzSubWinHomo<-c(0.02,0.06,0.125);
     intSubRatioHomo<-c(0.1,0.08,0.02);   
     maxMZDiffHomo<-0.1                   
     } else {}    
  } else if (isTOFTailing==TRUE) {
     intBase<-100
     intValidThreshold<-0.05
     mzWinTail<-1
  }  else {}

  isFindPeakPairs<-TRUE                           
  chargeVec<-c(1.00336,0.50168,0.33445,0.25084);   
  mzTolChargeVec<-c(10e-6,10e-6,10e-6,10e-6);    

  intRatioRangeChargeMat<-matrix(c(0.15,0.4,0.25,0.45,0.3,0.5,0.35,0.55),2,4)
  intSaturated<-2.5E+06                             # Upper limit of dynamic range.
  intRatioC12_1stTag<-2.5;                      
  intRatioC13_to_1stISO<-c(0.5,0.5,0.5,0.5);      
                                                 
  intRatio1stIso_C13_C12<-c(0.5,1.5);             
                                                  
  mzTolIso<-c(10e-6,10e-6,10e-6,10e-6);            
  mzTolFixLev3<-10e-6;                             
 
  mzTolDimer<-10e-6

  mz_tol_dt<-10E-6

  H_deconv<-1.007276
  mz_tol_deconv<-20E-6
  
  gpMethod<-2
  if (gpMethod==1)
  {
     mzTolGroup<-20e-6;                            
     intRatioPeakOverValley=3.0;                      
  } else {
     mzWinGroup<-1
     mzTolGroup<-20e-6
  }

  isMZBackground<-TRUE       
  mzTolBackground<-20e-6
  mzFileBackground<-"mzBackground.csv"

  mzTolFilter<-20e-6
  mzFilterFile<-"mzFilter.csv"

  mzCutoffFile<-"mzCutoff.csv";                   
  
  mz_tol_ctfilter<-20e-6

  minIntensity<-3000                      # The threshold of exported pairs that is used in TOF version
  minRatio<-0.1                         
  maxRatio<-10                          
  if (isFT==TRUE) {minSN<-20}             # The threshold of exported pairs that is used in FT version



  #-------------------------------End of parameter table------------------------------------------#

  # creating saving folder
  if ( ! file.exists(dataSaveFolder) )
  {
     dir.create(dataSaveFolder,showWarnings=FALSE, recursive=TRUE)
  }
  # creating saving folder for debug files
  if ( ! file.exists(dataSaveFolderDebug))
  {
     dir.create(dataSaveFolderDebug,showWarnings=FALSE, recursive=TRUE)
  }

  con <- file(paste(dataSaveFolderDebug,"screenlog.txt",sep=""))
  sink(con, append=TRUE, split=TRUE);

  if (isFT==TRUE){
      print("",quote = FALSE)
      print("IsoMS FT version",quote = FALSE)
   } else {
      print("",quote = FALSE)
      print("IsoMS TOF version",quote = FALSE)
   }
  
     
  # list all csv file
  csvfiles<-list.files(path=convertedDataFolder,pattern="(.csv)$")
  for ( nf in 1:length(csvfiles) ) {
        
      # read data from each csv file
      SPFileFromDA<-csvfiles[nf]
      print("",quote = FALSE)
      print(paste(SPFileFromDA,"started at:",format(Sys.time(),"%Y-%B-%d %H:%M:%S",quote = FALSE)),quote = FALSE)
 
      finalResultFileName<-paste(dataSaveFolder,"IsoMS_results_",SPFileFromDA,sep="")
      pairsOnlyFewTimes<-paste(dataSaveFolder,"pairsOnlyShowFewTimes_",SPFileFromDA,sep="")
      if (isFT==TRUE)
      {
         badSpectrumLog<-paste(dataSaveFolderDebug,"badSpectrumLog_",SPFileFromDA,sep="")
         largeRatioLog <-paste(dataSaveFolderDebug,"largeRatioLog_",SPFileFromDA,sep="")
      }
      debugFileInFindPeakPairs<-paste(dataSaveFolderDebug,"debug_in_findPeakPairs_",SPFileFromDA,sep="")
      debugFileInCleanIsotope<-paste(dataSaveFolderDebug,"debug_in_cleanIsotope_",SPFileFromDA,sep="")
      debugFileInGroupPairs<-paste(dataSaveFolderDebug,"debug_in_gruopPairs_",SPFileFromDA,sep="")
            
      # read the data table exported from DataAnalysis
      oriCSVFile<-paste(dataFolder,SPFileFromDA,sep="")
      pk<-read.csv(oriCSVFile,header=T,sep=",")
      if (nrow(pk)==0) {stop("no data in original csv file")}

      # mz, rt window and intensity threshhold filtering
      indValid<-which(pk$rt>rtWinMin & pk$rt<rtWinMax & pk$mz>mzWinMin & pk$mz<mzWinMax & pk$maxo>intThreshold)
      pk<-pk[indValid,]
      if (nrow(pk)==0) {stop("no data after mz, rt window and intensity threshhold filtering");}

      if (isFT==TRUE)
      {
         #filter bad spectra
         pk<-removeBadSpectra(pk,minResRatio,badSpectrumLog,largeRatioLog)         
        if (isFTHomo==TRUE) {
         pk<-removeHomoPeaks(pk,mzWinHomo,mzSubWinHomo,intSubRatioHomo,maxMZDiffHomo)
        } 
      } else if (isTOFTailing==TRUE) {
         badSpectrumLog<-NA
         # remove tailing
         pk<-removeTailing(pk,intBase,intValidThreshold,mzWinTail)
      } else {badSpectrumLog<-NA}
      if (nrow(pk)==0) {stop("no data before find PeakPair!!");}
           
      # find peak pairs
      pairs<-findPeakPairsDupl(pk,chargeVec,mzTolChargeVec,intRatioRangeChargeMat,intSaturated,intRatioC12_1stTag,intRatioC13_to_1stISO,intRatio1stIso_C13_C12,mzTolIso,mzTolFixLev3,isFT,debugFileInFindPeakPairs);
      if (nrow(pairs)==0) {stop("no data after findPeakPairs");}
      rm(pk)

      # remove dimer
      nodimerPairs<-removeDimer(pairs,mzTolDimer); rm(pairs)
      if (nrow(nodimerPairs)==0) {stop("no data after removeDimer")}
           
      # clean derivatives
      nodtPairs<-cleanDerivatives(nodimerPairs,mz_tol_dt); rm(nodimerPairs)
      if (nrow(nodtPairs)==0) {stop("no data after cleanDerivatives")}
     
      # deconv 
      deconvPairs<-deconvPeaks(nodtPairs,H_deconv,mz_tol_deconv); rm(nodtPairs)
      if (nrow(deconvPairs)==0) {stop("no data after deconvPairs")}
           
      # grouping ==> removing the tailing ones
      if (gpMethod==1)
      {
         gpPairs<-groupPairs(deconvPairs,mzTolGroup,intRatioPeakOverValley,badSpectrumLog,debugFileInGroupPairs);
      } else {
         gpPairs<-groupPairsII(deconvPairs,mzWinMin,mzWinMax,mzWinGroup,mzTolGroup,badSpectrumLog,debugFileInGroupPairs,pairsOnlyFewTimes);
      }
      rm(deconvPairs)
      if (nrow(gpPairs)==0) {stop("no data after grouping")}
     
      # filter the background mz
      if (isMZBackground==TRUE){ 
         gpPairs<-mzFilterBackground(gpPairs,mzFileBackground,mzTolBackground);
      } else {
         # use mzFilter.R
         if (file.exists(mzFilterFile)){
            gpPairs<-mzFilter(gpPairs,mzFilterFile,mzTolFilter);
         } else {
            print(paste("warning: mzFilterFile is not found! ==> ",mzFilterFile,quote = FALSE),quote = FALSE)
         }
      }

      # filtering based on ratio & intensity 
      indFiltered<-which(gpPairs$int_light>=minIntensity & gpPairs$ratio_light_to_heavy>=minRatio & gpPairs$ratio_light_to_heavy<=maxRatio)
      pk.flt<-gpPairs[indFiltered, ]; rm(gpPairs)
      pk.flt<-pk.flt[order(pk.flt$nCharge,pk.flt$nTag,decreasing=FALSE),]

      # filter 2charge-2tag
      pk.flt<-ctFilter(pk.flt,mz_tol_ctfilter)
            
      # format output
      # sample, scanNum, rt, mz_light, mz_heavy, distance, int_light, int_heavy, ratio_light_to_heavy [,sn_light, sn_heavy], nCharge, nTag, pairLev, isSaturated
      nLen<-nrow(pk.flt);
      pkFormated<-vector(mode="character",length=(nLen+1))
      if (isFT==TRUE)
      {
        pkFormated[1]<-"sample,scanNum,rt,mz_light,mz_heavy,distance,int_light,int_heavy,ratio_light_to_heavy, sn_light, sn_heavy, nCharge, nTag, pairLev,isSaturated"
        pkFormated[2:(nLen+1)]<-sprintf("%2d, %5d, %0.2f, %0.6f, %0.6f, %0.6f, %0.2E, %0.2E, %0.2f, %0.4f, %0.4f, %2d, %2d, %2d, %2d",pk.flt[,1],pk.flt[,2],pk.flt[,3],pk.flt[,4],pk.flt[,5],pk.flt[,6],pk.flt[,7],pk.flt[,8],pk.flt[,9],pk.flt[,10],pk.flt[,11],pk.flt[,12],pk.flt[,13],pk.flt[,14],pk.flt[,15])
      } else {
        pkFormated[1]<-"sample,scanNum,rt,mz_light,mz_heavy,distance,int_light,int_heavy,ratio_light_to_heavy, nCharge, nTag, pairLev,isSaturated"
        pkFormated[2:(nLen+1)]<-sprintf("%2d,%5d, %0.2f, %0.6f, %0.6f, %0.6f, %0.2E, %0.2E, %0.2f, %2d, %2d, %2d, %2d",pk.flt[,1],pk.flt[,2],pk.flt[,3],pk.flt[,4],pk.flt[,5],pk.flt[,6],pk.flt[,7],pk.flt[,8],pk.flt[,9],pk.flt[,10],pk.flt[,11],pk.flt[,12],pk.flt[,13])
      }
      write.table(pkFormated, file=finalResultFileName,col.names=FALSE,row.names=FALSE,quote=FALSE);
      
      print("", quote=FALSE)
      print(paste(SPFileFromDA," was done at:",format(Sys.time(),"%Y-%B-%d %H:%M:%S")),quote = FALSE);
      print("",quote=FALSE)
      rm(SPFileFromDA)
  }
  print("",quote = FALSE)
  print(paste("All data have been Done at: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S",quote=FALSE)),quote = FALSE);
  sink()
